%===============================================================================
% The Quantized Kernel LMS Algorithm
% contact: cedric.richard@unice.fr
%
% Chen B., Zhao S., Zhu P., and Principe J. ?Quantized kernel least mean square algorithm?. IEEE
% Transactions on Neural Networks and Learning Systems, Vol. 23, No. 1, pp. 22?32, Jan. 2012.
%
%
%
% function [err,ndict]=qklms(v,d,mu,tresh,ker,p1);
%
% inputs of the function
% v     : matrix of the inputs of the filter, each row corresponding
%         to the input values at a given time instant.
% d     : row vector of desired outputs.
%
% mu    : step-size of Q-KLMS
%
% tresh : novelty threshold in [0,1]. If the novelty coefficient (coherence)
%         of an input v(i) is larger than tresh, it is inserted into the
%         dictionary. A consequence is the increase of the order of the filter.
%         
% ker   : kernel, e.g., 'poly', 'gauss', ...
% p1    : parameter of the kernel, e.g., std of the rbf kernel
%
% output of the function
% err   : a priori output estimation error
% ndict : size of the dictionary
%===============================================================================


function [err,ndict]=qklms(v,d,mu,tresh,ker,p1);

%====================================================================
% Initialization
%
% dict   : dictionary
% modict : modulus of the elements v(i) of the dictionary
%====================================================================
dic=v(1,:);
tdict=1;
k=kernel(ker,dic,dic,p1);
modict=sqrt(k);

% Initialization of the weigth vector
alpha=d(1)/k;

% Filtering process
i=1;
while i<size(v,1)
    i=i+1;
    
    % Mapping of input v(i) into the feature space
    b=v(i,:);
    kv=kernel(ker,dic,b,p1);
    k=kernel(ker,b,b,p1);
    C=kv./(sqrt(k)*modict);
    
    % If v(i) is sufficiently novel, that is, the coherence of 
    % the dictionary remains smaller than the threshold tresh, v(i)
    % is inserted into the dictionary and the size of the weight
    % vector alpha is increased by 1.
    [val,j0]=max(abs(C));
    if (val<tresh)  
        dic=[dic;b];
        modict=[modict;sqrt(k)];
        tdict=[tdict;i];
        alpha=[alpha;0];
        j0=length(alpha);
    end
    
    % Iteration of APA in the feature space
    h=kernel(ker,dic,v(i,:),p1)';
    err(i)=d(i)-h*alpha;
    alpha(j0)=alpha(j0)+mu*err(i);
end

tdict;
ndict=length(tdict);



